function  [breakpoints, diff_parameterization_coeff_filtred]=lib_chanpoint_detect(sol2, N, degree, param_method) 
 
 % Function that detects borders of segments in given parameterization
 % coefficients.
 %
 % INPUT:
 % sol2 - parameterization coefficients
 % N - length of signal y
 % degree - polynom degree
 % param_method - structure of paramters
 % - threshold - threshold used for breakpoint detection
 % - window_length - window length of median filter
 % - p - type of norm. Default p=2 -> Euclidean norm. 
 % - proximity - smallest proximity of two breakpoints. Default proximity=2
 %
 % OUTPUT:
 % breakpoints - detected breakpoints
 % diff_parameterization_coeff_filtred - vector of differences in
 % filtered Euclidean distance in parameterization coefficients
 %-------------------------------------------------------------------------

 threshold=param_method.threshold;
 window_length=param_method.window_length;
 parameterization_coeff_sol=reshape(sol2,N,degree); % reshape given parameters
 matrix_of_differences=zeros(N-1,degree);
 matrix_of_weighted_differences=zeros(N-1,degree);


 % differences in paramteres
 figure('Name','Differences in parameters..','NumberTitle','off')
 for i=1:degree
     matrix_of_differences(:,i)=diff(parameterization_coeff_sol(:,i));
     subplot (degree+2,1,i) 
     plot(matrix_of_differences(:,i))
     title (['Differences in parameter x_', num2str(i-1)])
 end

  % Euclidean distance in differneces of parameters
 for i=1:degree
     matrix_of_weighted_differences(:,i)=abs((matrix_of_differences(:,i))/(max(abs(matrix_of_differences(:,i)))));
     disp(['alpha_', num2str(i-1) , ': ', num2str(1/(max(abs(matrix_of_differences(:,i)))))])
 end
 
 if ~isfield(param_method, 'p'), param_method.p=2 ; end
 p=param_method.p;
 diff_parameterization_coeff=(sum(matrix_of_weighted_differences.^p,2)).^(1/p);

 subplot(degree+2,1,degree+1)
 plot(diff_parameterization_coeff)
 title ('Euclidean distance of differences in parameters (d)')

% Median filter of Euclidean distance of differences of parameters
subplot (degree+2,1,degree+2) 
median_diff=medfilt1(diff_parameterization_coeff,window_length);
diff_parameterization_coeff_filtred=diff_parameterization_coeff-median_diff;
plot (diff_parameterization_coeff_filtred)
title (['Subtracted median filtering of d (with window of length ', num2str(window_length), ')'])

 
% other distances
 figure('Name','Differences in parameters - norms','NumberTitle','off')
 colors = 'gcmyr';
 pom=0;
 norms=[1,2,100]; % define p
 for j=1:length(norms)
     i=norms(j);
     pom=pom+1;
     diff_parameterization_coeff_p=(sum(matrix_of_weighted_differences.^i,2)).^(1/i);
     plot(diff_parameterization_coeff_p, colors(pom))
     hold on
     title (['l_p norm - distance of differences in parameters (d)'])
 end
legend ('p=1','p=2','p=100')

% Breakpoint detection 
breakpoints=find(abs(diff_parameterization_coeff_filtred)>threshold); % find breakpoints, when the differences are larger than threshold
breakpoints=[0;breakpoints;N]; % defines start position of first segment minus 1, and end position of last segment

% delete breakpoint which are next to each other, leaves the one with
% larger value in difference vector
if ~isfield(param_method, 'proximity'), param_method.proximity=2 ; end
proximity=param_method.proximity;
diff_breakpoints=(diff(breakpoints)<proximity);
pozice=find(diff_breakpoints==1);    
while sum(diff_breakpoints)>0
%     for i=1:length(diff_breakpoints)-1
    for j=1:length(pozice)
        i=pozice(j);
        if (diff_breakpoints(i))==1 & breakpoints(i)<N & breakpoints(i)>1
            br_1=diff_parameterization_coeff_filtred(breakpoints(i));
            br_2=diff_parameterization_coeff_filtred(breakpoints(i+1));
            if abs(br_1)>abs(br_2)
                breakpoints_pom=[breakpoints(1:i);breakpoints(i+2:end)];
            else
                breakpoints_pom=[breakpoints(1:i-1);breakpoints(i+1:end)];

            end
            break
        end

    end
    breakpoints=breakpoints_pom;
    diff_breakpoints=(diff(breakpoints)<proximity);
    pozice=find(diff_breakpoints==1); 
end